@@ -26,6 +26,7 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -59,6 +60,7 @@
 #include "ctkdisplayconfig.h"
 #include "ctkserver.h"
 #include "ctkecc.h"
+#include "ctkgridlicense.h"
 #include "ctkvdpau.h"
 
 #include "ctkappprofile.h"
@@ -71,6 +73,7 @@
 #include "common-utils.h"
 #include "query-assign.h"
 
+#include "opengl_loading.h"
 
 /* column enumeration */
 
@@ -727,13 +730,13 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         }
 
 
-        /* GLX Information */
+        /* Graphics Information */
 
         child = ctk_glx_new(screen_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_glx_create_help(tag_table, CTK_GLX(child));
             add_page(child, help, ctk_window, &iter, NULL,
-                     "OpenGL/GLX Information", NULL, ctk_glx_probe_info, NULL);
+                     "Graphics Information", NULL, ctk_glx_probe_info, NULL);
         }
 
 
@@ -1054,6 +1057,19 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                  NULL, NULL, NULL);
     }
 
+    /* Manage GRID License Information */
+    for (node = system->targets[GPU_TARGET]; node; node = node->next) {
+        widget = ctk_manage_grid_license_new(node->t, ctk_config);
+        if (widget) {
+            help = ctk_manage_grid_license_create_help(tag_table,
+                                                       CTK_MANAGE_GRID_LICENSE(widget));
+            add_page(widget, help, ctk_window, NULL, NULL, "Manage License",
+                     NULL, ctk_manage_grid_license_start_timer,
+                     ctk_manage_grid_license_stop_timer);
+            break; /* only add the page once */
+        }
+    }
+
     /* nvidia-settings configuration */
 
     add_page(GTK_WIDGET(ctk_window->ctk_config),
@@ -1329,6 +1345,20 @@ static GtkWidget *create_quit_dialog(CtkWindow *ctk_window)
 
 static void save_settings_and_exit(CtkWindow *ctk_window)
 {
+#ifdef CTK_GTK3
+    GdkDisplay *gdk_display;
+    Display *display;
+    GdkScreen *gdk_screen;
+
+    if (dGL.glXMakeContextCurrent) {
+        gdk_screen = gtk_window_get_screen(GTK_WINDOW(ctk_window));
+        gdk_display = gdk_screen_get_display(gdk_screen);
+        display = gdk_x11_display_get_xdisplay(gdk_display);
+        dGL.glXMakeContextCurrent(display, 0, 0, 0);
+    }
+
+#endif
+
     add_special_config_file_attributes(ctk_window);
     gtk_main_quit();
 }
@@ -1444,7 +1474,7 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
         int display_id = pData[i+1];
         char *logName;
         char *typeBaseName;
-        char *typeIdName;
+        char *randrName;
         GtkWidget *widget;
         gchar *title;
         CtkEvent *ctk_event;
@@ -1484,19 +1514,19 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
             logName = NULL;
         }
         ret = NvCtrlGetStringAttribute(target,
-                                       NV_CTRL_STRING_DISPLAY_NAME_TYPE_ID,
-                                       &typeIdName);
+                                       NV_CTRL_STRING_DISPLAY_NAME_RANDR,
+                                       &randrName);
         if (ret != NvCtrlSuccess) {
-            typeIdName = NULL;
+            randrName = NULL;
         }
 
-        if (!logName && !typeIdName) {
+        if (!logName && !randrName) {
             title = g_strdup_printf("DPY-%d - (Unknown)", display_id);
         } else {
-            title = g_strdup_printf("%s - (%s)", typeIdName, logName);
+            title = g_strdup_printf("%s - (%s)", randrName, logName);
         }
         free(logName);
-        free(typeIdName);
+        free(randrName);
 
         /* Create the page for the display */
         ctk_event = CTK_EVENT(ctk_event_new(target));
